package com.fundi.framework.common.instance;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.displaylist.DisplayList;
import com.fundi.framework.common.displaylist.ListFieldColor;
import com.fundi.framework.common.navigation.SerializableNavigationItem;
import com.fundi.framework.common.nl1.Messages;

@XmlRootElement
public class SerializableAppInstance implements Serializable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;

	private String instanceType = null;
	
	private int timeout = 60 * 60 * 12;  //12 hours as seconds.
	private String title;
	private String currentUserid = ""; //$NON-NLS-1$
	private String timezone = AppInstance.timezoneClient;  //Default to client's timezone.
	private boolean regularlyTestingConnections = false;
	private int connectionsTestInterval = AppInstance.defaultSystemsRefreshInterval;
	private boolean rememberConnectionSettings = false;
	private Boolean showConnectionProfilesAtStartup = true;  //Added later - so Boolean instead of boolean so we can handle null.
	private boolean restoreEditorsAtStartup = true;
	private boolean keepListKeysAtFront = false;
	private boolean savingListLayouts = true;
	private ListFieldColor listKeyBackground = new ListFieldColor(DisplayList.colorKeyDefaultRed, DisplayList.colorKeyDefaultGreen, DisplayList.colorKeyDefaultBlue);
	private ListFieldColor compDiffBackground = new ListFieldColor(DisplayList.colorDifferenceDefaultRed, DisplayList.colorDifferenceDefaultGreen, DisplayList.colorDifferenceDefaultBlue);
	private ListFieldColor compSepBackground = new ListFieldColor(DisplayList.colorSeparatorDefaultRed, DisplayList.colorSeparatorDefaultGreen, DisplayList.colorSeparatorDefaultBlue);
	
	private ArrayList<AppUser> knownUsers = new ArrayList<AppUser>();
	
	private ArrayList<SerializableNavigationItem> navigationGroups = new ArrayList<SerializableNavigationItem>();
	
	private ArrayList<String> appParms = new ArrayList<String>();
	
	private String traceFilePath = ""; //$NON-NLS-1$
	
	private Boolean useIBMExplorerConnectionDefns = false;  //Added later - so Boolean instead of boolean so we can handle a null.
	
	public SerializableAppInstance() {
		
		super();
	}

	public String getCurrentUserid() {
		
		return currentUserid;
	}
	
	public String getCurrentPassword() {
		
		Iterator<AppUser> itUsr;
		AppUser user;
		
		for (itUsr = knownUsers.iterator(); itUsr.hasNext();) {
			user = itUsr.next();
			if (user.getUserid().equals(currentUserid)) {
				return user.getPassword();
			}
		}
		
		return ""; //$NON-NLS-1$
	}
	
	public void setCurrentUserid(String userid) {
		
		currentUserid = userid;
	}
	
	public ArrayList<AppUser> getKnownUsers() {
		
		return knownUsers;
	}
	
	public void addUser(AppUser user) {
		
		knownUsers.add(user);
	}

	public String getInstanceType() {
		
		return instanceType;
	}

	public void setInstanceType(String instanceType) {
		
		this.instanceType = instanceType;
	}

	public int getTimeout() {
		
		return timeout;
	}

	public void setTimeout(int timeout) {
		
		this.timeout = timeout;
	}

	public String getTitle() {
		
		return title;
	}

	public void setTitle(String title) {
		
		this.title = title;
	}

	public String getTimezone() {
		
		return timezone;
	}

	public void setTimezone(String timezone) {
		
		this.timezone = timezone;
	}

	public boolean regularlyTestingConnections() {
		
		return regularlyTestingConnections;
	}

	public void setRegularlyTestingConnections(boolean regularlyTestingConnections) {
		
		this.regularlyTestingConnections = regularlyTestingConnections;
	}

	public int getConnectionsTestInterval() {
		
		return connectionsTestInterval;
	}

	public void setConnectionsTestInterval(int connectionsTestInterval) {
		
		this.connectionsTestInterval = connectionsTestInterval;
	}

	public boolean restoreEditorsAtStartup() {
		
		return restoreEditorsAtStartup;
	}

	public void setRestoreEditorsAtStartup(boolean restoreEditorsAtStartup) {
		
		this.restoreEditorsAtStartup = restoreEditorsAtStartup;
	}

	public boolean keepListKeysAtFront() {
		
		return keepListKeysAtFront;
	}
	
	public void setKeepListKeysAtFront(boolean keepListKeysAtFront) {
		
		this.keepListKeysAtFront = keepListKeysAtFront;
	}

	public boolean savingListLayouts() {
		
		return savingListLayouts;
	}

	public void setSavingListLayouts(boolean savingListLayouts) {
		
		this.savingListLayouts = savingListLayouts;
	}

	public ListFieldColor getListKeyBackground() {
		
		if (listKeyBackground == null) {
			 listKeyBackground = new ListFieldColor(DisplayList.colorKeyDefaultRed, DisplayList.colorKeyDefaultGreen, DisplayList.colorKeyDefaultBlue);
		}
		return listKeyBackground;
	}

	public void setListKeyBackground(ListFieldColor listKeyBackground) {
		
		this.listKeyBackground = listKeyBackground;
	}
	
	public ListFieldColor getComparatorDiffBackground() {
		
		if (compDiffBackground == null) {
			compDiffBackground = new ListFieldColor(DisplayList.colorDifferenceDefaultRed, DisplayList.colorDifferenceDefaultGreen, DisplayList.colorDifferenceDefaultBlue);
		}
		return compDiffBackground;
	}

	public void setComparatorDiffBackground(ListFieldColor compDiffBackground) {
		
		this.compDiffBackground = compDiffBackground;
	}
	
	public ListFieldColor getComparatorSepBackground() {
		
		if (compSepBackground == null) {
			compSepBackground = new ListFieldColor(DisplayList.colorSeparatorDefaultRed, DisplayList.colorSeparatorDefaultGreen, DisplayList.colorSeparatorDefaultBlue);
		}
		return compSepBackground;
	}

	public void setComparatorSepBackground(ListFieldColor compSepBackground) {
		
		this.compSepBackground = compSepBackground;
	}
	
	public boolean rememberConnectionSettings() {
		
		return rememberConnectionSettings;
	}

	public void setRememberConnectionSettings(boolean rememberConnectionSettings) {
		
		this.rememberConnectionSettings = rememberConnectionSettings;
	}
	
	public void setShowConnectionProfilesAtStartup(boolean showAtStartup) {
		
		showConnectionProfilesAtStartup = showAtStartup;
	}
	
	public boolean showConnectionProfilesAtStartup() {
		
		if (showConnectionProfilesAtStartup == null) {
			showConnectionProfilesAtStartup = true;
		}
		return showConnectionProfilesAtStartup;
	}

	public ArrayList<SerializableNavigationItem> getNavigationGroups() {
		
		if (navigationGroups == null) {
			navigationGroups = new ArrayList<SerializableNavigationItem>();
		}
		navigationGroups.trimToSize();
		
		return navigationGroups;
	}

	public void addNavigationGroup(SerializableNavigationItem group) {
		
		navigationGroups.add(group);
	}

	public String getTraceFilePath() {
		
		return traceFilePath;
	}

	public void setTraceFilePath(String traceFilePath) {
		
		this.traceFilePath = traceFilePath;
	}
	
	public void setUseIBMExplorerConnectionDefns(boolean useIBMExplorer) {
		
		useIBMExplorerConnectionDefns = useIBMExplorer;
	}
	
	public boolean useIBMExplorerConnectionDefns() {
		
		if (useIBMExplorerConnectionDefns == null) {
			useIBMExplorerConnectionDefns = false;
		}
		return useIBMExplorerConnectionDefns;
	}

	public String getApplicationParm(String parmName, String defaultValue) {
		
		int idx;
		String value = null; 
		String parm;
		
		if (appParms == null) {
			appParms = new ArrayList<String>();
		}
		
		for (idx = 0; idx < appParms.size(); idx++) {
			parm = appParms.get(idx);
			if (parm.startsWith(parmName + "=")) { //$NON-NLS-1$
				value = parm.substring(parmName.length() + 1);
				break;
			}
		}
		
		if (value == null && defaultValue != null) {
			setApplicationParm(parmName, defaultValue);
			value = defaultValue;
		}
		
		return value;
	}
	
	public void setApplicationParm(String parmName, String value) {
		
		int idx;
		boolean fnd = false;
		String parm;
		String newParm = parmName + "=" + value; //$NON-NLS-1$
		
		if (appParms == null) {
			appParms = new ArrayList<String>();
		}
		
		for (idx = 0; idx < appParms.size(); idx++) {
			parm = appParms.get(idx);
			if (parm.startsWith(parmName + "=")) { //$NON-NLS-1$
				fnd = true;
				appParms.set(idx, newParm);
				break;
			}
		}
		
		if (!fnd) {
			appParms.add(newParm);
		}
	}
}
